function [pirf, jirf, wirf, dirf, fevd1, fevd2, fevd3, fevd4, sho_p, sho_j, sho_w, sho_d, A0, ind, d] = GIRF_FEVD_2019(sd, aa, qd, qv, s1, s2, s3, s4, vd, om,...
    YS, HOR, N, L, cors, NMCInteg, h, epsilon, SC,G1)

% This function estimates generalised impulse response functions following
% Koop et al (1996) and uses the algorithm in Rubio-Ramirez et al (2010) to
% identify structural shocks. Code is based on Baumeister and Peersman
% (2013) JAE and Baumeister and Peersman (2013) AEJ:Macroeconomics. In
% particular after computing the GIRFs, we compute structural variance
% deocmpositions in the frequency domain as in Benati and Mumtaz (2007) and
% FEVDs as in Baumeister and Peersman (2013) AEJ.

% Michael Ellington 12/01/2017

% Step 1: Simulate VAR elements into the future
%        1) Simulate VAR covariance matrix
% Shocks to A_{t} matrix
% NOTE: since we are simulating the responses in each point in time A_{t} =
% A at each observation.
aa_e = real(sqrtm([s1 zeros(1,2) zeros(1,3) zeros(1,4); zeros(2,1) s2 zeros(2,3) zeros(2,4);...
    zeros(3,3) s3 zeros(3,4); zeros(4,1), zeros(4,2) zeros(4,3) s4]))*randn((N*(N-1))/2,HOR+L+1);
% This puts shocks in the block diagonal matrix S (see Eq(9) in paper)
% Now simulate elements of matrix through the horizon
for jj = 1:HOR+L+1
    aa(:,:,jj+1) = aa(:,:,jj) + aa_e(:,jj);
end

% Shocks to the H_{t} matrix 
om_e = diag(vd)*randn(N,HOR+L+1);
% Simulate H_{t} matrix over the horizon
om = log(om);
for jj = 1:HOR+L+1
    om(:,:,jj+1) = om(:,:,jj) + om_e(:,jj);
end
om = exp(om);
% Compute the VARs reduced-form covariance matrix
VAR=zeros(N,N,HOR+L+1);
for jj = 1:HOR+L+1
    iA = inv(chofac(N, aa(:,:,jj)));
    VAR(:,:,jj) = iA*diag(om(:,:,jj))*iA'; % \Omega_{t}
end
VARC = VAR(:,:,1);
    
if G1==0
% Shocks to Q_{t} matrix
qd_e = diag(qv)*rand(N*(1+N*L),HOR+L+1);
qd = log(qd);
for jj = 1:HOR+L+1
    qd(:,:,jj+1) = qd(:,:,jj) + qd_e(:,jj);
end
qd = exp(qd);
end
% 2) Simulate the theta matrix (i.e. the \Beta_{t})
SI = sd;
B = zeros(N,N*L+1,HOR+L+1);
jj = 2;
trial=0;
maxtrial = 50;
while jj <= HOR+L+1
    
    if G1==1
    dd = sd(:,:,jj-1) + real(mysqrt_varm(qd))*randn(N*(1+N*L),1);
    elseif G1==0
    dd = sd(:,:,jj-1) + real(mysqrt_varm(diag(qd(:,jj))))*randn(N*(1+N*L),1); 
    end
    b = [dd(1:1+N*L,1)'; dd((1+N*L)+1:2*(1+N*L),1)';...
        dd(2*(1+N*L)+1:3*(1+N*L),1)'; dd(3*(1+N*L)+1:4*(1+N*L),1)'; dd(4*(1+N*L)+1:5*(1+N*L),1)'];
        
        if SC == 0
        sd(:,:,jj) = dd;
        B(:,:,jj) = b;
        jj = jj+1;
        else
            if max(abs(varroots(L,N,b))) < 1
                sd(:,:,jj) = dd;
                B(:,:,jj) = b;
                jj = jj+1;
                trial = 0;
            elseif max(abs(varroots(L,N,b))) > 1 && trial < maxtrial
                trial = trial + 1;
            elseif max(abs(varroots(L,N,b))) > 1 && trial== maxtrial
               pirf=[]; jirf=[]; wirf=[]; dirf=[];...
               fevd1=[]; fevd2=[]; fevd3=[]; fevd4=[]; sho_p=[]; sho_j=[]; sho_w=[]; sho_d=[]; A0=[]; ind=0; d=0;
               return
            end
        end
end
% Lines 46:72 simulate the shocks hitting the beta matrix we then put the
% shocked coefficients into the matrix B which has equations in the rows
% and parameters in the columns. the pages represent the "forecast horizon"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 2: GIRFs                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Non-Accumulated Responses
irp1 = zeros(NMCInteg, HOR);
irp2 = irp1;
irp3 = irp1;
irp4 = irp1;
irp5=irp1;

irj1 = irp1;
irj2 = irp1;
irj3 = irp1;
irj4 = irp1;
irj5=irp1;

irw1 = irp1;
irw2 = irp1;
irw3 = irp1;
irw4 = irp1;
irw5=irp1;

ird1 = irp1;
ird2 = irp1;
ird3 = irp1;
ird4 = irp1;
ird5=irp1;


% FEVDs
dep1 = zeros(NMCInteg,HOR);
dep2 = dep1;
dep3 = dep1;
dep4 = dep1;
dep5 = dep1;

dej1 = dep1;
dej2 = dep1;
dej3 = dep1;
dej4 = dep1;
dej5 = dep1;

dew1 = dep1;
dew2 = dep1;
dew3 = dep1;
dew4 = dep1;
dew5 = dep1;

ded1 = dep1;
ded2 = dep1;
ded3 = dep1;
ded4 = dep1;
ded5 = dep1;

sho_p = zeros(NMCInteg,1);
sho_j = zeros(NMCInteg,1);
sho_w = zeros(NMCInteg,1);
sho_d = zeros(NMCInteg,1);

struct_shocks = zeros(NMCInteg,N);

tel = 0; count = 0; maxcount = 2500;
while tel < NMCInteg && count < maxcount
    count = count + 1;
    a0 = impact(VARC,N); % compute the contemporaneous impact matrix
    % using the Rubio-Ramires et al. (2010) algorithm 
    
    Y_B = [YS zeros(N, HOR+(L-1)-L)]; % if YS(:,tt-L:tt-1) and tt=5 to start YS(:,1:4) if N= 3, L=4
    % then Y_B is 3x24 matrix with zeros in each row from column 5
    % "HOR+(L-1)-L" places a matrix of zeros with N rows through the
    % forecast horizon.
    Y_SUP = Y_B;
    Y_DEM = Y_B;
    Y_MOP = Y_B;
    Y_MOD = Y_B;
    SHOCKS = randn(N, HOR);
    % Calculate the impact of the shocks:
    Y_B(:,L+1) = a0*SHOCKS(:,1); % Y_B is Y_baseline where no shock occurs
    Y_SUP(:,L+1) = a0*[1+SHOCKS(1,1) SHOCKS(2:5,1)']'; % epsilon_1,i +1 is the first shock
    % a0 is 3x3 SHOCKS vector is 3x1 therefore the value in Y_SUP(:,L+1)
    % is the dot product of a0 and the shock vector where the first element
    % in SHOCKS vector is 1 + the random draw
    Y_DEM(:,L+1) = a0*[SHOCKS(1,1) 1+SHOCKS(2,1) SHOCKS(3:5,1)']';
    Y_MOP(:,L+1) = a0*[SHOCKS(1:2,1)' 1+SHOCKS(3,1) SHOCKS(4:5,1)']';
    Y_MOD(:,L+1) = a0*[SHOCKS(1:3,1)' 1+SHOCKS(4,1) SHOCKS(5,1)]';
    Y_MMM(:,L+1) = a0*[SHOCKS(1:4,1)' 1+SHOCKS(5,1)]';
    for tt = 2+L:HOR+L
        shocks = mysqrt_varm(VAR(:,:,tt-L))*SHOCKS(:,tt-L);
        % Take the eigenvalue eigenvector deocmposition of the variance
        % covariance matrix. VCV matrix is symmetric and positive definite
        % therefore  asquare root always exists. Taking the square root of
        % the eigenvalues always exists. eigenvalue eigenvector decomposit
        % -ion implies [Q,T] = eig(A), Q contains eigenvectors, and T contains the eigenvalues 
        % A=Q*T*Q' R=diag(sqrt(diag(T)); takes the square root of the eigenvalues
        % thus A^{0.5} = Q*R*Q';
        Y_B(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_B(:,tt-L:tt-1)))] + shocks;
        % if N=3, L=4 B(:,:,tt-L) is 3x13. let vec1 = 
        % [1; myvec(myfliplr(Y_B(:,tt-L:tt-1)))] denote the vector of lagged
        % dependent values at this point in time as well as the structural
        % impact matrix. myfliplr is essentially the same as fliplr and
        % flips the columns from left to right meaning that the structural
        % impact matrix are in rows 2:4 (essentially Y(t-1), row 1 is the 
        % constant. rows 5:7 Y(t-2), are observed Y values, 8:10 Y(t-3),
        % 11:13 Y(t-4). myvec does the same as A=A(:) stacking columns into
        % a vector. The dot product of B*vec1 is 3x1 adding shocks which is
        % also 3x1 results in Y_B(:,tt) being 3x1. This is the conditional
        % forecast at horizon tt under the scenario that no shock hits the
        % system.
        Y_SUP(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_SUP(:,tt-L:tt-1)))] + shocks;
        Y_DEM(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_DEM(:,tt-L:tt-1)))] + shocks;
        Y_MOP(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_MOP(:,tt-L:tt-1)))] + shocks;
        Y_MOD(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_MOD(:,tt-L:tt-1)))] + shocks;
        Y_MMM(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_MMM(:,tt-L:tt-1)))] + shocks;
        % The above are the conditional forecasts of variables in the VAR
        % when a shock hits the system.
    end
        Y_B = Y_B(:,L+1:end);
        Y_SUP = Y_SUP(:,L+1:end);
        Y_DEM = Y_DEM(:,L+1:end);
        Y_MOP = Y_MOP(:,L+1:end);
        Y_MOD = Y_MOD(:,L+1:end);
        Y_MMM = Y_MMM(:,L+1:end);
        
        % IRF is the differences between the conditional forecast of VAR
        % when a shock hits and the conditional forecast of when the shock
        % does not hit.
        
        irf1 = Y_SUP - Y_B; % shock 1
        irf2 = Y_DEM - Y_B; % shock 2
        irf3 = Y_MOP - Y_B; % shock 3
        irf4 = Y_MOD - Y_B; % shock 4
        irf5 = Y_MMM - Y_B; % shock 5
        
        % accumulated shocks
        ir1 = cumsum(irf1,2);
        ir2 = cumsum(irf2,2);
        ir3 = cumsum(irf3,2);
        ir4 = cumsum(irf4,2);
        ir5 = cumsum(irf5,2);
        
        irf = zeros(N,N,HOR); % NOT ACCUMULATED 
        ir = zeros(N,N,HOR); % ACCUMULATED
        for xx = 1:HOR
            irf(:,:,xx) = [irf1(:,xx) irf2(:,xx) irf3(:,xx) irf4(:,xx) irf5(:,xx)];
            ir(:,:,xx) = [ir1(:,xx) ir2(:,xx) ir3(:,xx) ir4(:,xx) ir5(:,xx)];
        end
        clear irf1 irf2 irf3 irf4 irf5 ir1 ir2 ir3 ir4 ir5
        % irf and ir contain all impulse response functions. NxN represents
        % number of equations by number of shocks. Columns in irf and ir
        % track the shock, column 1 tracks shock 1. Pages track the impulse
        % horizon. E.g. If i want the response of variable 1 to shock 1 
        % across the impulse horizon, i want irf(1,1,:)
        
        % Check Sign Restrictions
        ncorr = N*N; % number of sign correlations. if N=3 then ncorr = 9
        co = zeros(cors+h,ncorr); % if N=3, cors && h = 1 => co is 2x9
        
        for ss = 1:N
            for kl = 1:cors+h
                co(kl,1+N*(ss-1)) = ir(1,ss,kl)/ir(2,ss,cors);
                co(kl,2+N*(ss-1)) = ir(2,ss,kl)/ir(2,ss,cors);
                co(kl,3+N*(ss-1)) = ir(3,ss,kl)/ir(2,ss,cors);
                co(kl,4+N*(ss-1)) = ir(4,ss,kl)/ir(2,ss,cors);
                co(kl,5+N*(ss-1)) = ir(5,ss,kl)/ir(2,ss,cors);
            end
        end
        % This normalises the correlations. If N=4 of the shocks col 1 normalises
        % shock 1 to variable 2. col 4 normalises shock 2 to variable 2 and
        % col 7 normalises shock 3 to variable 2.
        % we are normalising on VACANCIES, so sign normalisation takes
        % this into account.
        s1ident = 0; s2ident = 0; s3ident = 0; s4ident = 0; s5ident = 0;
        if h == 1
            % Productivity shock
            if co(cors,1)>=0 && co(cors,2)>=0 && co(cors,3)<=0 && co(cors,4)>=0 && co(cors,5)<=0
                s1ident = 1000;
            end
            if co(cors,6)>=0 && co(cors,7)>=0 && co(cors,8)<=0 && co(cors,9)>=0 && co(cors,10)<=0
                s2ident = 1000;
            end
            if co(cors,11)>=0 && co(cors,12)>=0 && co(cors,13)<=0 && co(cors,14)>=0 && co(cors,15)<=0
                s3ident = 1000;
            end
            if co(cors,16)>=0 && co(cors,17)>=0 && co(cors,18)<=0 && co(cors,19)>=0 && co(cors,20)<=0
                s4ident = 1000;
            end
            if co(cors,21)>=0 && co(cors,22)>=0 && co(cors,23)<=0 && co(cors,24)>=0 && co(cors,25)<=0
                s5ident = 1000;
            end

            % Job separation shock
            if  co(cors,2)>=0 && co(cors,3)>=0 && co(cors,4)<=0
                s1ident = 100;
            end
            if co(cors,7)>=0 && co(cors,8)>=0 && co(cors,9)<=0 
                s2ident = 100;
            end
            if co(cors,12)>=0 && co(cors,13)>=0 && co(cors,14)<=0
                s3ident = 100;
            end
            if co(cors,17)>=0 && co(cors,18)>=0 && co(cors,19)<=0
                s4ident = 100;
            end
            if  co(cors,22)>=0 && co(cors,23)>=0 && co(cors,24)<=0
                s5ident = 100;
            end
            
            % Wage Shock
            if  co(cors,2)>=0 && co(cors,3)<=0 && co(cors,4)<=0
                s1ident = 10;
            end
            if co(cors,7)>=0 && co(cors,8)<=0 && co(cors,9)<=0 
                s2ident = 10;
            end
            if co(cors,12)>=0 && co(cors,13)<=0 && co(cors,14)<=0
                s3ident = 10;
            end
            if co(cors,17)>=0 && co(cors,18)<=0 && co(cors,19)<=0
                s4ident = 10;
            end
            if  co(cors,22)>=0 && co(cors,23)<=0 && co(cors,24)<=0
                s5ident = 10;
            end
            % Demand shock
            if co(cors,2)>=0 && co(cors,3)<=0 && co(cors,4)>=0 && co(cors,5)>=0
                s1ident = 1;
            end
            if co(cors,7)>=0 && co(cors,8)<=0 && co(cors,9)>=0 && co(cors,10)>=0
                s2ident = 1;
            end
            if co(cors,12)>=0 && co(cors,13)<=0 && co(cors,14)>=0 && co(cors,15)>=0
                s3ident = 1;
            end
            if co(cors,17)>=0 && co(cors,18)<=0 && co(cors,19)>=0 && co(cors,20)>=0
                s4ident = 1;
            end
            
            if co(cors,22)>=0 && co(cors,23)<=0 && co(cors,24)>=0 && co(cors,25)>=0
                s5ident = 1;
            end
            
        end
            identot = s1ident + s2ident + s3ident + s4ident + s5ident;
            clear co
            if identot == 1111
                tel = tel+1;
                % tel tracks the distribution with which we calculate the
                % mean/median response of our endogenous variables over the
                % accepted draws.
                strucshock = structural_shocks(a0,N,epsilon);
                decomp = vardec(N,HOR,irf); % computes the forecast error variance decomposition
                
                % shock 1
                if s1ident == 1000 % then productivity shock
                    irp1(tel,1:HOR) = irf(1,1,1:HOR); 
                    irp2(tel,1:HOR) = irf(2,1,1:HOR); 
                    irp3(tel,1:HOR) = irf(3,1,1:HOR); 
                    irp4(tel,1:HOR) = irf(4,1,1:HOR); 
                    irp5(tel,1:HOR) = irf(5,1,1:HOR);
                    dep1(tel,1:HOR) = decomp(1,1,1:HOR);
                    dep2(tel,1:HOR) = decomp(2,1,1:HOR);
                    dep3(tel,1:HOR) = decomp(3,1,1:HOR);
                    dep4(tel,1:HOR) = decomp(4,1,1:HOR);
                    dep5(tel,1:HOR) = decomp(5,1,1:HOR);
                    sho_p(tel,1) = strucshock(1,1);
                    a_p(tel,:)=a0(:,1);
                end
                if s1ident == 100 % then jobseparation shock
                    irj1(tel,1:HOR) = irf(1,1,1:HOR); 
                    irj2(tel,1:HOR) = irf(2,1,1:HOR); 
                    irj3(tel,1:HOR) = irf(3,1,1:HOR); 
                    irj4(tel,1:HOR) = irf(4,1,1:HOR); 
                    irj5(tel,1:HOR) = irf(5,1,1:HOR); 
                    dej1(tel,1:HOR) = decomp(1,1,1:HOR);
                    dej2(tel,1:HOR) = decomp(2,1,1:HOR);
                    dej3(tel,1:HOR) = decomp(3,1,1:HOR);
                    dej4(tel,1:HOR) = decomp(4,1,1:HOR);
                    dej5(tel,1:HOR) = decomp(5,1,1:HOR);
                    sho_j(tel,1) = strucshock(1,1);
                    a_j(tel,:)=a0(:,1);
                end
                if s1ident == 10 % then  wage shock
                    irw1(tel,1:HOR) = irf(1,1,1:HOR); 
                    irw2(tel,1:HOR) = irf(2,1,1:HOR); 
                    irw3(tel,1:HOR) = irf(3,1,1:HOR); 
                    irw4(tel,1:HOR) = irf(4,1,1:HOR);
                    irw5(tel,1:HOR) = irf(5,1,1:HOR);
                    dew1(tel,1:HOR) = decomp(1,1,1:HOR);
                    dew2(tel,1:HOR) = decomp(2,1,1:HOR);
                    dew3(tel,1:HOR) = decomp(3,1,1:HOR);
                    dew4(tel,1:HOR) = decomp(4,1,1:HOR);
                    dew5(tel,1:HOR) = decomp(5,1,1:HOR);
                    sho_w(tel,1) = strucshock(1,1);
                    a_w(tel,:)=a0(:,1);
                end
                if s1ident == 1 % then demand shock
                    ird1(tel,1:HOR) = irf(1,1,1:HOR); 
                    ird2(tel,1:HOR) = irf(2,1,1:HOR); 
                    ird3(tel,1:HOR) = irf(3,1,1:HOR); 
                    ird4(tel,1:HOR) = irf(4,1,1:HOR);
                    ird5(tel,1:HOR) = irf(5,1,1:HOR);
                    ded1(tel,1:HOR) = decomp(1,1,1:HOR);
                    ded2(tel,1:HOR) = decomp(2,1,1:HOR);
                    ded3(tel,1:HOR) = decomp(3,1,1:HOR);
                    ded4(tel,1:HOR) = decomp(4,1,1:HOR);
                    ded5(tel,1:HOR) = decomp(5,1,1:HOR);
                    sho_d(tel,1) = strucshock(1,1);
                    a_d(tel,:)=a0(:,1);
                end
                
                
                % shock 2
                if s2ident == 1000 % then productivity shock
                    irp1(tel,1:HOR) = irf(1,2,1:HOR); 
                    irp2(tel,1:HOR) = irf(2,2,1:HOR); 
                    irp3(tel,1:HOR) = irf(3,2,1:HOR); 
                    irp4(tel,1:HOR) = irf(4,2,1:HOR); 
                    irp5(tel,1:HOR) = irf(5,2,1:HOR);
                    dep1(tel,1:HOR) = decomp(1,2,1:HOR);
                    dep2(tel,1:HOR) = decomp(2,2,1:HOR);
                    dep3(tel,1:HOR) = decomp(3,2,1:HOR);
                    dep4(tel,1:HOR) = decomp(4,2,1:HOR);
                    dep5(tel,1:HOR) = decomp(5,2,1:HOR);
                    sho_p(tel,1) = strucshock(1,2);
                    a_p(tel,:)=a0(:,2);
                end
                if s2ident == 100 % then jobseparation shock
                    irj1(tel,1:HOR) = irf(1,2,1:HOR); 
                    irj2(tel,1:HOR) = irf(2,2,1:HOR); 
                    irj3(tel,1:HOR) = irf(3,2,1:HOR); 
                    irj4(tel,1:HOR) = irf(4,2,1:HOR); 
                    irj5(tel,1:HOR) = irf(5,2,1:HOR); 
                    dej1(tel,1:HOR) = decomp(1,2,1:HOR);
                    dej2(tel,1:HOR) = decomp(2,2,1:HOR);
                    dej3(tel,1:HOR) = decomp(3,2,1:HOR);
                    dej4(tel,1:HOR) = decomp(4,2,1:HOR);
                    dej5(tel,1:HOR) = decomp(5,2,1:HOR);
                    sho_j(tel,1) = strucshock(1,2);
                    a_j(tel,:)=a0(:,2);
                end
                if s2ident == 10 % then  wage shock
                    irw1(tel,1:HOR) = irf(1,2,1:HOR); 
                    irw2(tel,1:HOR) = irf(2,2,1:HOR); 
                    irw3(tel,1:HOR) = irf(3,2,1:HOR); 
                    irw4(tel,1:HOR) = irf(4,2,1:HOR);
                    irw5(tel,1:HOR) = irf(5,2,1:HOR);
                    dew1(tel,1:HOR) = decomp(1,2,1:HOR);
                    dew2(tel,1:HOR) = decomp(2,2,1:HOR);
                    dew3(tel,1:HOR) = decomp(3,2,1:HOR);
                    dew4(tel,1:HOR) = decomp(4,2,1:HOR);
                    dew5(tel,1:HOR) = decomp(5,2,1:HOR);
                    sho_w(tel,1) = strucshock(1,2);
                    a_w(tel,:)=a0(:,2);
                end
                if s2ident == 1 % then demand shock
                    ird1(tel,1:HOR) = irf(1,2,1:HOR); 
                    ird2(tel,1:HOR) = irf(2,2,1:HOR); 
                    ird3(tel,1:HOR) = irf(3,2,1:HOR); 
                    ird4(tel,1:HOR) = irf(4,2,1:HOR);
                    ird5(tel,1:HOR) = irf(5,2,1:HOR);
                    ded1(tel,1:HOR) = decomp(1,2,1:HOR);
                    ded2(tel,1:HOR) = decomp(2,2,1:HOR);
                    ded3(tel,1:HOR) = decomp(3,2,1:HOR);
                    ded4(tel,1:HOR) = decomp(4,2,1:HOR);
                    ded5(tel,1:HOR) = decomp(5,2,1:HOR);
                    sho_d(tel,1) = strucshock(1,2);
                    a_d(tel,:)=a0(:,2);
                end
                % shock 3
                if s3ident == 1000 % then productivity shock
                    irp1(tel,1:HOR) = irf(1,3,1:HOR); 
                    irp2(tel,1:HOR) = irf(2,3,1:HOR); 
                    irp3(tel,1:HOR) = irf(3,3,1:HOR); 
                    irp4(tel,1:HOR) = irf(4,3,1:HOR); 
                    irp5(tel,1:HOR) = irf(5,3,1:HOR);
                    dep1(tel,1:HOR) = decomp(1,3,1:HOR);
                    dep2(tel,1:HOR) = decomp(2,3,1:HOR);
                    dep3(tel,1:HOR) = decomp(3,3,1:HOR);
                    dep4(tel,1:HOR) = decomp(4,3,1:HOR);
                    dep5(tel,1:HOR) = decomp(5,3,1:HOR);
                    sho_p(tel,1) = strucshock(1,3);
                    a_p(tel,:)=a0(:,3);
                end
                if s3ident == 100 % then jobseparation shock
                    irj1(tel,1:HOR) = irf(1,3,1:HOR); 
                    irj2(tel,1:HOR) = irf(2,3,1:HOR); 
                    irj3(tel,1:HOR) = irf(3,3,1:HOR); 
                    irj4(tel,1:HOR) = irf(4,3,1:HOR); 
                    irj5(tel,1:HOR) = irf(5,3,1:HOR); 
                    dej1(tel,1:HOR) = decomp(1,3,1:HOR);
                    dej2(tel,1:HOR) = decomp(2,3,1:HOR);
                    dej3(tel,1:HOR) = decomp(3,3,1:HOR);
                    dej4(tel,1:HOR) = decomp(4,3,1:HOR);
                    dej5(tel,1:HOR) = decomp(5,3,1:HOR);
                    sho_j(tel,1) = strucshock(1,3);
                    a_j(tel,:)=a0(:,3);
                end
                if s3ident == 10 % then  wage shock
                    irw1(tel,1:HOR) = irf(1,3,1:HOR); 
                    irw2(tel,1:HOR) = irf(2,3,1:HOR); 
                    irw3(tel,1:HOR) = irf(3,3,1:HOR); 
                    irw4(tel,1:HOR) = irf(4,3,1:HOR);
                    irw5(tel,1:HOR) = irf(5,3,1:HOR);
                    dew1(tel,1:HOR) = decomp(1,3,1:HOR);
                    dew2(tel,1:HOR) = decomp(2,3,1:HOR);
                    dew3(tel,1:HOR) = decomp(3,3,1:HOR);
                    dew4(tel,1:HOR) = decomp(4,3,1:HOR);
                    dew5(tel,1:HOR) = decomp(5,3,1:HOR);
                    sho_w(tel,1) = strucshock(1,3);
                    a_w(tel,:)=a0(:,3);
                end
                if s3ident == 1 % then demand shock
                    ird1(tel,1:HOR) = irf(1,3,1:HOR); 
                    ird2(tel,1:HOR) = irf(2,3,1:HOR); 
                    ird3(tel,1:HOR) = irf(3,3,1:HOR); 
                    ird4(tel,1:HOR) = irf(4,3,1:HOR);
                    ird5(tel,1:HOR) = irf(5,3,1:HOR);
                    ded1(tel,1:HOR) = decomp(1,3,1:HOR);
                    ded2(tel,1:HOR) = decomp(2,3,1:HOR);
                    ded3(tel,1:HOR) = decomp(3,3,1:HOR);
                    ded4(tel,1:HOR) = decomp(4,3,1:HOR);
                    ded5(tel,1:HOR) = decomp(5,3,1:HOR);
                    sho_d(tel,1) = strucshock(1,3);
                    a_d(tel,:)=a0(:,3);
                end
                % shock 4
                if s4ident == 1000 % then productivity shock
                    irp1(tel,1:HOR) = irf(1,4,1:HOR); 
                    irp2(tel,1:HOR) = irf(2,4,1:HOR); 
                    irp3(tel,1:HOR) = irf(3,4,1:HOR); 
                    irp4(tel,1:HOR) = irf(4,4,1:HOR); 
                    irp5(tel,1:HOR) = irf(5,4,1:HOR);
                    dep1(tel,1:HOR) = decomp(1,4,1:HOR);
                    dep2(tel,1:HOR) = decomp(2,4,1:HOR);
                    dep3(tel,1:HOR) = decomp(3,4,1:HOR);
                    dep4(tel,1:HOR) = decomp(4,4,1:HOR);
                    dep5(tel,1:HOR) = decomp(5,4,1:HOR);
                    sho_p(tel,1) = strucshock(1,4);
                    a_p(tel,:)=a0(:,4);
                end
                if s4ident == 100 % then jobseparation shock
                    irj1(tel,1:HOR) = irf(1,4,1:HOR); 
                    irj2(tel,1:HOR) = irf(2,4,1:HOR); 
                    irj3(tel,1:HOR) = irf(3,4,1:HOR); 
                    irj4(tel,1:HOR) = irf(4,4,1:HOR); 
                    irj5(tel,1:HOR) = irf(5,4,1:HOR); 
                    dej1(tel,1:HOR) = decomp(1,4,1:HOR);
                    dej2(tel,1:HOR) = decomp(2,4,1:HOR);
                    dej3(tel,1:HOR) = decomp(3,4,1:HOR);
                    dej4(tel,1:HOR) = decomp(4,4,1:HOR);
                    dej5(tel,1:HOR) = decomp(5,4,1:HOR);
                    sho_j(tel,1) = strucshock(1,4);
                    a_j(tel,:)=a0(:,4);
                end
                if s4ident == 10 % then  wage shock
                    irw1(tel,1:HOR) = irf(1,4,1:HOR); 
                    irw2(tel,1:HOR) = irf(2,4,1:HOR); 
                    irw3(tel,1:HOR) = irf(3,4,1:HOR); 
                    irw4(tel,1:HOR) = irf(4,4,1:HOR);
                    irw5(tel,1:HOR) = irf(5,4,1:HOR);
                    dew1(tel,1:HOR) = decomp(1,4,1:HOR);
                    dew2(tel,1:HOR) = decomp(2,4,1:HOR);
                    dew3(tel,1:HOR) = decomp(3,4,1:HOR);
                    dew4(tel,1:HOR) = decomp(4,4,1:HOR);
                    dew5(tel,1:HOR) = decomp(5,4,1:HOR);
                    sho_w(tel,1) = strucshock(1,4);
                    a_w(tel,:)=a0(:,4);
                end
                if s4ident == 1 % then demand shock
                    ird1(tel,1:HOR) = irf(1,4,1:HOR); 
                    ird2(tel,1:HOR) = irf(2,4,1:HOR); 
                    ird3(tel,1:HOR) = irf(3,4,1:HOR); 
                    ird4(tel,1:HOR) = irf(4,4,1:HOR);
                    ird5(tel,1:HOR) = irf(5,4,1:HOR);
                    ded1(tel,1:HOR) = decomp(1,4,1:HOR);
                    ded2(tel,1:HOR) = decomp(2,4,1:HOR);
                    ded3(tel,1:HOR) = decomp(3,4,1:HOR);
                    ded4(tel,1:HOR) = decomp(4,4,1:HOR);
                    ded5(tel,1:HOR) = decomp(5,4,1:HOR);
                    sho_d(tel,1) = strucshock(1,4);
                    a_d(tel,:)=a0(:,4);
                end
                % shock 5
                if s5ident == 1000 % then productivity shock
                    irp1(tel,1:HOR) = irf(1,5,1:HOR); 
                    irp2(tel,1:HOR) = irf(2,5,1:HOR); 
                    irp3(tel,1:HOR) = irf(3,5,1:HOR); 
                    irp4(tel,1:HOR) = irf(4,5,1:HOR); 
                    irp5(tel,1:HOR) = irf(5,5,1:HOR);
                    dep1(tel,1:HOR) = decomp(1,5,1:HOR);
                    dep2(tel,1:HOR) = decomp(2,5,1:HOR);
                    dep3(tel,1:HOR) = decomp(3,5,1:HOR);
                    dep4(tel,1:HOR) = decomp(4,5,1:HOR);
                    dep5(tel,1:HOR) = decomp(5,5,1:HOR);
                    sho_p(tel,1) = strucshock(1,5);
                    a_p(tel,:)=a0(:,5);
                end
                if s5ident == 100 % then jobseparation shock
                    irj1(tel,1:HOR) = irf(1,5,1:HOR); 
                    irj2(tel,1:HOR) = irf(2,5,1:HOR); 
                    irj3(tel,1:HOR) = irf(3,5,1:HOR); 
                    irj4(tel,1:HOR) = irf(4,5,1:HOR); 
                    irj5(tel,1:HOR) = irf(5,5,1:HOR); 
                    dej1(tel,1:HOR) = decomp(1,5,1:HOR);
                    dej2(tel,1:HOR) = decomp(2,5,1:HOR);
                    dej3(tel,1:HOR) = decomp(3,5,1:HOR);
                    dej4(tel,1:HOR) = decomp(4,5,1:HOR);
                    dej5(tel,1:HOR) = decomp(5,5,1:HOR);
                    sho_j(tel,1) = strucshock(1,5);
                    a_j(tel,:)=a0(:,5);
                end
                if s5ident == 10 % then  wage shock
                    irw1(tel,1:HOR) = irf(1,5,1:HOR); 
                    irw2(tel,1:HOR) = irf(2,5,1:HOR); 
                    irw3(tel,1:HOR) = irf(3,5,1:HOR); 
                    irw4(tel,1:HOR) = irf(4,5,1:HOR);
                    irw5(tel,1:HOR) = irf(5,5,1:HOR);
                    dew1(tel,1:HOR) = decomp(1,5,1:HOR);
                    dew2(tel,1:HOR) = decomp(2,5,1:HOR);
                    dew3(tel,1:HOR) = decomp(3,5,1:HOR);
                    dew4(tel,1:HOR) = decomp(4,5,1:HOR);
                    dew5(tel,1:HOR) = decomp(5,5,1:HOR);
                    sho_w(tel,1) = strucshock(1,5);
                    a_w(tel,:)=a0(:,5);
                end
                if s5ident == 1 % then demand shock
                    ird1(tel,1:HOR) = irf(1,5,1:HOR); 
                    ird2(tel,1:HOR) = irf(2,5,1:HOR); 
                    ird3(tel,1:HOR) = irf(3,5,1:HOR); 
                    ird4(tel,1:HOR) = irf(4,5,1:HOR);
                    ird5(tel,1:HOR) = irf(5,5,1:HOR);
                    ded1(tel,1:HOR) = decomp(1,5,1:HOR);
                    ded2(tel,1:HOR) = decomp(2,5,1:HOR);
                    ded3(tel,1:HOR) = decomp(3,5,1:HOR);
                    ded4(tel,1:HOR) = decomp(4,5,1:HOR);
                    ded5(tel,1:HOR) = decomp(5,5,1:HOR);
                    sho_d(tel,1) = strucshock(1,5);
                    a_d(tel,:)=a0(:,5);
                end
                
                
                
                struct_shocks(tel,:) = strucshock;
                
                if s1ident>0 && s2ident>0 && s3ident>0 && s4ident>0
                    a_1(tel,:)=a0(:,5);
                elseif s1ident>0 && s3ident>0 && s4ident>0 && s5ident>0
                    a_1(tel,:)=a0(:,2);
                elseif s1ident>0 && s2ident>0 && s4ident>0 && s5ident>0
                    a_1(tel,:)=a0(:,3);
                elseif s1ident>0 && s2ident>0 && s3ident>0 && s5ident>0
                    a_1(tel,:)=a0(:,4);
                end
                
            end
            
end
% [count tel]
% line 391 finishes computing the GIRFs satisying the 100 sign restrictions
% imposed on the model.
if count == maxcount
    % empty variables and start again
    pirf=[]; jirf=[]; wirf=[]; dirf=[];...
        fevd1=[]; fevd2=[]; fevd3=[]; fevd4=[]; sho_p=[]; sho_j=[]; sho_w=[]; sho_d=[]; A0=[]; ind=0; d=0;
    return
else ind = 1; d = 1;
end


% normalisation of signs to compute distributions 
t1 = irp1(:,1); % prod shock to prod at time 1
t2 = irj2(:,1); % JS shock to v at time 1
t3 = irw2(:,1); % wage shock to v at time 1
t4 = ird2(:,1); % demand shock to v at time 1

vs1 = zeros(tel,1); vs2 = vs1; vs3 = vs1; vs4 = vs1; 
for jj = 1:tel
    if t1(jj,1) > 0
        vs1(jj,1) = 1;
    else
        vs1(jj,1) = -1;
    end
    if t2(jj,1) > 0
        vs2(jj,1) = 1;
    else
        vs2(jj,1) = -1;
    end
    if t3(jj,1) >0
        vs3(jj,1) = 1;
    else
        vs3(jj,1) = -1;
    end
    if t4(jj,1)>0
        vs4(jj,1) = 1;
    else
        vs4(jj,1) = -1;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GIRFs                                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NON-ACCUMULATED
for jj = 1:HOR
    irp1(:,jj) = vs1.*irp1(:,jj);
    irp2(:,jj) = vs1.*irp2(:,jj);
    irp3(:,jj) = vs1.*irp3(:,jj);
    irp4(:,jj) = vs1.*irp4(:,jj);
    irp5(:,jj) = vs1.*irp5(:,jj);
end
for jj = 1:HOR
    irj1(:,jj) = vs2.*irj1(:,jj);
    irj2(:,jj) = vs2.*irj2(:,jj);
    irj3(:,jj) = vs2.*irj3(:,jj);
    irj4(:,jj) = vs2.*irj4(:,jj);
    irj5(:,jj) = vs2.*irj5(:,jj);
end
for jj = 1:HOR
    irw1(:,jj) = vs3.*irw1(:,jj);
    irw2(:,jj) = vs3.*irw2(:,jj);
    irw3(:,jj) = vs3.*irw3(:,jj);
    irw4(:,jj) = vs3.*irw4(:,jj);
    irw5(:,jj) = vs3.*irw5(:,jj);
end
for jj = 1:HOR
    ird1(:,jj) = vs4.*ird1(:,jj);
    ird2(:,jj) = vs4.*ird2(:,jj);
    ird3(:,jj) = vs4.*ird3(:,jj);
    ird4(:,jj) = vs4.*ird4(:,jj);
    ird5(:,jj) = vs4.*ird5(:,jj);
end
% Take mean or median of the distribution


% GIRFs to productivity shock
girfp1 = mean(irp1); % response of y to productivity shock
girfp2 = mean(irp2); % response of v to productivity shock
girfp3 = mean(irp3); % response of u to productivity shock
girfp4 = mean(irp4); % response of w to productivity shock 
girfp5 = mean(irp5); % response of pi to productivity shock

% GIRFs to job separation shock
girfj1 = mean(irj1); % response of y to job separation shock
girfj2 = mean(irj2); % response of v to job separation shock
girfj3 = mean(irj3); % response of u to job separation shock
girfj4 = mean(irj4); % response of w to job separation shock
girfj5 = mean(irj5); % response of pi to job separation shock

% GIRFs to wage shock
girfw1 = mean(irw1); % response of y to wage
girfw2 = mean(irw2); % response of v to wage
girfw3 = mean(irw3); % response of u to wage
girfw4 = mean(irw4); % response of w to wage
girfw5 = mean(irw5); % response of pi to wage

% GIRFs to demand shock
girfd1 = mean(ird1); % response of y to wage
girfd2 = mean(ird2); % response of v to wage
girfd3 = mean(ird3); % response of u to wage
girfd4 = mean(ird4); % response of w to wage
girfd5 = mean(ird5); % response of pi to wage

% Put into matrices for the output 
pirf = [girfp1' girfp2' girfp3' girfp4' girfp5'];
jirf = [girfj1' girfj2' girfj3' girfj4' girfj5'];
wirf = [girfw1' girfw2' girfw3' girfw4' girfw5'];
dirf = [girfd1' girfd2' girfd3' girfd4' girfd5'];

% Put mean of decompositions into matrix for output
fevd1 = [mean(dep1)' mean(dep2)' mean(dep3)' mean(dep4)' mean(dep5)'];
fevd2 = [mean(dej1)' mean(dej2)' mean(dej3)' mean(dej4)' mean(dej5)'];
fevd3 = [mean(dew1)' mean(dew2)' mean(dew3)' mean(dew4)' mean(dew5)'];
fevd4 = [mean(ded1)' mean(ded2)' mean(ded3)' mean(ded4)' mean(ded5)'];

sho_p = vs1.*sho_p;
sho_j = vs2.*sho_j;
sho_w = vs3.*sho_w;
sho_d = vs4.*sho_d;

sho_p = mean(sho_p);
sho_j = mean(sho_j);
sho_w = mean(sho_w);
sho_d = mean(sho_d);

for i=1:N
    a_w(:,i)=vs3.*a_w(:,i);
    a_p(:,i)=vs1.*a_p(:,i);
    a_j(:,i)=vs2.*a_j(:,i);
    a_d(:,i)=vs4.*a_d(:,i);
end
a_p=mean(a_p);
a_j=mean(a_j);
a_w=mean(a_w);
a_d=mean(a_d);
a_1=mean(a_1);

A0 = [a_p', a_j', a_w', a_d', a_1'];

    

