% GIRF PARTIAL IDENTIFICATION OF LIQUIDITY SHOCK
% We assume that Liquidity shocks increases GDP, inflation unconstrained,
% money increase and interest rate is 0 on impact only. use deterministic
% rotation matrix to compute.
function [sirf, dirf, iirf, mirf, sirfacc, dirfacc, iirfacc, mirfacc,...
    fevd1, fevd2, fevd3, fevd4, sho_liq, sho_d, sho_s, sho_mp, A0_impact, B, ind, d] =...
    GIRF_ZRpartial_FEVD(sd, aa,qd, s1, s2, s3, vd, om,...
    YS, HOR, N, L, cors, NMCInteg, h, epsilon, SC)

% This function estimates generalised impulse response functions following
% Koop et al (1996) and uses the algorithm in Rubio-Ramirez et al (2010) to
% identify structural shocks. Code is based on Baumeister and Peersman
% (2013) JAE and Baumeister and Peersman (2013) AEJ:Macroeconomics. In
% particular after computing the GIRFs, we compute structural variance
% deocmpositions in the frequency domain as in Benati and Mumtaz (2007) and
% FEVDs as in Baumeister and Peersman (2013) AEJ.



% Step 1: Simulate VAR elements into the future
%        1) Simulate VAR covariance matrix
% Shocks to A_{t} matrix
% NOTE: since we are simulating the responses in each point in time A_{t} =
% A at each observation.
aa_e = real(sqrtm([s1 zeros(1,2) zeros(1,3); zeros(2,1) s2 zeros(2,3);...
    zeros(3,3) s3]))*randn((N*(N-1))/2,HOR+L+1);
% This puts shocks in the block diagonal matrix S (see Eq(9) in paper)
% Now simulate elements of matrix through the horizon
for jj = 1:HOR+L+1
    aa(:,:,jj+1) = aa(:,:,jj) + aa_e(:,jj);
end

% Shocks to the H_{t} matrix 
om_e = diag(vd)*randn(N,HOR+L+1);
% Simulate H_{t} matrix over the horizon
om = log(om);
for jj = 1:HOR+L+1
    om(:,:,jj+1) = om(:,:,jj) + om_e(:,jj);
end
om = exp(om);
% Compute the VARs reduced-form covariance matrix
VAR=zeros(N,N,HOR+L+1);
for jj = 1:HOR+L+1
    iA = inv(chofac(N, aa(:,:,jj)));
    VAR(:,:,jj) = iA*diag(om(:,:,jj))*iA'; % \Omega_{t}
end
VARC = VAR(:,:,1);
% 2) Simulate the theta matrix (i.e. the \Beta_{t})
SI = sd;
B = zeros(N,N*L+1,HOR+L+1);
jj = 2;
trial=0;
maxtrial = 50;
while jj <= HOR+L+1
    
    dd = sd(:,:,jj-1) + real(mysqrt_varm(qd))*randn(N*(1+N*L),1); 
    b = [dd(1:1+N*L,1)'; dd((1+N*L)+1:2*(1+N*L),1)';...
        dd(2*(1+N*L)+1:3*(1+N*L),1)'; dd(3*(1+N*L)+1:4*(1+N*L),1)'];
        
        if SC == 0
        sd(:,:,jj) = dd;
        B(:,:,jj) = b;
        jj = jj+1;
        else
            if max(abs(varroots(L,N,b))) < 1
                sd(:,:,jj) = dd;
                B(:,:,jj) = b;
                jj = jj+1;
                trial = 0;
            elseif max(abs(varroots(L,N,b))) > 1 && trial < maxtrial
                trial = trial + 1;
            elseif max(abs(varroots(L,N,b))) > 1 && trial== maxtrial
               sirf=[]; dirf=[]; iirf=[]; mirf=[]; sirfacc=[]; dirfacc=[]; iirfacc=[]; mirfacc=[];...
               fevd1=[]; fevd2=[]; fevd3=[]; fevd4=[]; sho_liq=[];...
               sho_d=[];  sho_s=[]; sho_mp=[]; A0_impact=[]; B=[]; ind=0; d=0;
               return
            end
        end
end
% Lines 46:72 simulate the shocks hitting the beta matrix we then put the
% shocked coefficients into the matrix B which has equations in the rows
% and parameters in the columns. the pages represent the "forecast horizon"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Step 2: GIRFs                                                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Non-Accumulated Responses
irs1 = zeros(NMCInteg, HOR);
irs2 = irs1;
irs3 = irs1;
irs4 = irs1;

ird1 = irs1;
ird2 = irs1;
ird3 = irs1;
ird4 = irs1;

iri1 = irs1;
iri2 = irs1;
iri3 = irs1;
iri4 = irs1;

irm1 = irs1;
irm2 = irs1;
irm3 = irs1;
irm4 = irs1;
% Accumulated Responses
irs1acc = zeros(NMCInteg, HOR);
irs2acc = irs1acc;
irs3acc = irs1acc;
irs4acc = irs1acc;

ird1acc = irs1acc;
ird2acc = irs1acc;
ird3acc = irs1acc;
ird4acc = irs1acc;

iri1acc = irs1acc;
iri2acc = irs1acc;
iri3acc = irs1acc;
iri4acc = irs1acc;

irm1acc = irs1acc;
irm2acc = irs1acc;
irm3acc = irs1acc;
irm4acc = irs1acc;
% FEVDs
des1 = zeros(NMCInteg,HOR);
des2 = des1;
des3 = des1;
des4 = des1;

ded1 = des1;
ded2 = des1;
ded3 = des1;
ded4 = des1;

dei1 = des1;
dei2 = des1;
dei3 = des1;
dei4 = des1;

dem1 = des1;
dem2 = des1;
dem3 = des1;
dem4 = des1;

sho_liq = zeros(NMCInteg,1);
sho_s = sho_liq;
sho_d = sho_liq;
sho_mp = sho_liq;

A0_sup = zeros(NMCInteg,N);
A0_dem = A0_sup;
A0_mop = A0_sup;
A0_mod = A0_sup;

tel = 0; count = 0; maxcount = 5000;
while tel < NMCInteg && count < maxcount
    count = count + 1;
    a0 = impact_ZR(VARC,N); % compute the contemporaneous impact matrix
    % using the Rubio-Ramires et al. (2010) algorithm 
    
    Y_B = [YS zeros(N, HOR+(L-1)-L)]; % if YS(:,tt-L:tt-1) and tt=5 to start YS(:,1:4) if N= 3, L=4
    % then Y_B is 3x24 matrix with zeros in each row from column 5
    % "HOR+(L-1)-L" places a matrix of zeros with N rows through the
    % forecast horizon.
    Y_SUP = Y_B;
    Y_DEM = Y_B;
    Y_MOP = Y_B;
    Y_MOD = Y_B;
    SHOCKS = randn(N, HOR);
    % Calculate the impact of the shocks:
    Y_B(:,L+1) = a0*SHOCKS(:,1); % Y_B is Y_baseline where no shock occurs
    Y_SUP(:,L+1) = a0*[1+SHOCKS(1,1) SHOCKS(2:4,1)']'; % epsilon_1,i +1 is the first shock
    % a0 is 3x3 SHOCKS vector is 3x1 therefore the value in Y_SUP(:,L+1)
    % is the dot product of a0 and the shock vector where the first element
    % in SHOCKS vector is 1 + the random draw
    Y_DEM(:,L+1) = a0*[SHOCKS(1,1) 1+SHOCKS(2,1) SHOCKS(3:4,1)']';
    Y_MOP(:,L+1) = a0*[SHOCKS(1:2,1)' 1+SHOCKS(3,1) SHOCKS(4,1)]';
    Y_MOD(:,L+1) = a0*[SHOCKS(1:3,1)' 1+SHOCKS(4,1)]';
    for tt = 2+L:HOR+L
        shocks = mysqrt_varm(VAR(:,:,tt-L))*SHOCKS(:,tt-L);
        % Take the eigenvalue eigenvector deocmposition of the variance
        % covariance matrix. VCV matrix is symmetric and positive definite
        % therefore  asquare root always exists. Taking the square root of
        % the eigenvalues always exists. eigenvalue eigenvector decomposit
        % -ion implies [Q,T] = eig(A), Q contains eigenvectors, and T contains the eigenvalues 
        % A=Q*T*Q' R=diag(sqrt(diag(T)); takes the square root of the eigenvalues
        % thus A^{0.5} = Q*R*Q';
        Y_B(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_B(:,tt-L:tt-1)))] + shocks;
        % if N=3, L=4 B(:,:,tt-L) is 3x13. let vec1 = 
        % [1; myvec(myfliplr(Y_B(:,tt-L:tt-1)))] denote the vector of lagged
        % dependent values at this point in time as well as the structural
        % impact matrix. myfliplr is essentially the same as fliplr and
        % flips the columns from left to right meaning that the structural
        % impact matrix are in rows 2:4 (essentially Y(t-1), row 1 is the 
        % constant. rows 5:7 Y(t-2), are observed Y values, 8:10 Y(t-3),
        % 11:13 Y(t-4). myvec does the same as A=A(:) stacking columns into
        % a vector. The dot product of B*vec1 is 3x1 adding shocks which is
        % also 3x1 results in Y_B(:,tt) being 3x1. This is the conditional
        % forecast at horizon tt under the scenario that no shock hits the
        % system.
        Y_SUP(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_SUP(:,tt-L:tt-1)))] + shocks;
        Y_DEM(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_DEM(:,tt-L:tt-1)))] + shocks;
        Y_MOP(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_MOP(:,tt-L:tt-1)))] + shocks;
        Y_MOD(:,tt) = B(:,:,tt-L)*[1; myvec(myfliplr(Y_MOD(:,tt-L:tt-1)))] + shocks;
        % The above are the conditional forecasts of variables in the VAR
        % when a shock hits the system.
    end
        Y_B = Y_B(:,L+1:end);
        Y_SUP = Y_SUP(:,L+1:end);
        Y_DEM = Y_DEM(:,L+1:end);
        Y_MOP = Y_MOP(:,L+1:end);
        Y_MOD = Y_MOD(:,L+1:end); % NxHOR matrices
        
        % IRF is the differences between the conditional forecast of VAR
        % when a shock hits and the conditional forecast of when the shock
        % does not hit.
        
        irf1 = Y_SUP - Y_B; % shock 1
        irf2 = Y_DEM - Y_B; % shock 2
        irf3 = Y_MOP - Y_B; % shock 3
        irf4 = Y_MOD - Y_B; % shock 4
        
        % accumulated shocks
        ir1 = cumsum(irf1,2);
        ir2 = cumsum(irf2,2);
        ir3 = cumsum(irf3,2);
        ir4 = cumsum(irf4,2);
        
        irf = zeros(N,N,HOR); % NOT ACCUMULATED 
        ir = zeros(N,N,HOR); % ACCUMULATED
        for xx = 1:HOR
            irf(:,:,xx) = [irf1(:,xx) irf2(:,xx) irf3(:,xx) irf4(:,xx)];
            ir(:,:,xx) = [ir1(:,xx) ir2(:,xx) ir3(:,xx) ir4(:,xx)];
        end
        clear irf1 irf2 irf3 irf4 ir1 ir2 ir3 ir4
        % irf and ir contain all impulse response functions. NxN represents
        % number of equations by number of shocks. Columns in irf and ir
        % track the shock, column 1 tracks shock 1. Pages track the impulse
        % horizon. E.g. If i want the response of variable 1 to shock 1 
        % across the impulse horizon, i want irf(1,1,:)
        
        % Check Sign Restrictions
        ncorr = N*N; % number of sign correlations. if N=3 then ncorr = 9
        co = zeros(cors+h,ncorr); % if N=3, cors && h = 1 => co is 2x9
        
        for ss = 1:N
            for kl = 1:cors+h
                co(kl,1+N*(ss-1)) = ir(1,ss,kl)/ir(1,ss,cors);
                co(kl,2+N*(ss-1)) = ir(2,ss,kl)/ir(1,ss,cors);
                co(kl,3+N*(ss-1)) = ir(3,ss,kl)/ir(1,ss,cors);
                co(kl,4+N*(ss-1)) = ir(4,ss,kl)/ir(1,ss,cors);
            end
        end
        % This normalises the correlations. If N=3 of the shocks col 1 normalises
        % shock 1 to variable 1. col 4 normalises shock 2 to variable 1 and
        % col 7 normalises shock 3 to variable 1.
        % we are normalising on GDP growth, so sign normalisation takes
        % this into account.
        s1ident = 0; s2ident = 0; s3ident = 0; s4ident = 0;
        if h == 1
            % Stock Market Illiquidity Shock
            if co(cors,13)>=0 && co(cors,14)>=0 && co(cors,16)<=0
                s4ident = 1;
            end
            % BECAUSE OF ZERO RESTRICTION, ILLIQ SHOCK CAN ONLY BE IN COL 4
            % of A0
            
            % Monetary Policy shock
            if co(cors,1)>=0 && co(cors,2)>=0 && co(cors,3)<=0 && co(cors,4)<=0
                s1ident = 10;
            end
            if co(cors,5)>=0 && co(cors,6)>=0 && co(cors,7)<=0 && co(cors,8)<=0
                s2ident = 10;
            end
            if co(cors,9)>=0 && co(cors,10)>=0 && co(cors,11)<=0 && co(cors,12)<=0
                s3ident = 10;
            end 
        end
            identot = s1ident + s2ident + s3ident + s4ident;
            clear co
            if identot == 11;
                tel = tel+1;
                % tel tracks the distribution with which we calculate the
                % mean/median response of our endogenous variables over the
                % accepted draws.
                strucshock = structural_shocks(a0,N,epsilon);
                decomp = vardecomp(N,HOR,irf); % computes the forecast error variance decomposition
                
                % shock 1
                
                if s1ident == 10
                    iri1(tel,1:HOR) = irf(1,1,1:HOR); iri1acc(tel,1:HOR) = ir(1,1,1:HOR);
                    iri2(tel,1:HOR) = irf(2,1,1:HOR); iri2acc(tel,1:HOR) = ir(2,1,1:HOR);
                    iri3(tel,1:HOR) = irf(3,1,1:HOR); iri3acc(tel,1:HOR) = ir(3,1,1:HOR);
                    iri4(tel,1:HOR) = irf(4,1,1:HOR); iri4acc(tel,1:HOR) = ir(4,1,1:HOR);
                    dei1(tel,1:HOR) = decomp(1,1,1:HOR);
                    dei2(tel,1:HOR) = decomp(2,1,1:HOR);
                    dei3(tel,1:HOR) = decomp(3,1,1:HOR);
                    dei4(tel,1:HOR) = decomp(4,1,1:HOR);
                    sho_mp(tel,1) = strucshock(1,1);
                    A0_mop(tel,:) = a0(:,1);
                end
                
                
                if s2ident == 10
                    iri1(tel,1:HOR) = irf(1,2,1:HOR); iri1acc(tel,1:HOR) = ir(1,2,1:HOR);
                    iri2(tel,1:HOR) = irf(2,2,1:HOR); iri2acc(tel,1:HOR) = ir(2,2,1:HOR);
                    iri3(tel,1:HOR) = irf(3,2,1:HOR); iri3acc(tel,1:HOR) = ir(3,2,1:HOR);
                    iri4(tel,1:HOR) = irf(4,2,1:HOR); iri4acc(tel,1:HOR) = ir(4,2,1:HOR);
                    dei1(tel,1:HOR) = decomp(1,2,1:HOR);
                    dei2(tel,1:HOR) = decomp(2,2,1:HOR);
                    dei3(tel,1:HOR) = decomp(3,2,1:HOR);
                    dei4(tel,1:HOR) = decomp(4,2,1:HOR);
                    sho_mp(tel,1) = strucshock(1,2);
                    A0_mop(tel,:) = a0(:,2);
                end
                
                
                if s3ident == 10
                    iri1(tel,1:HOR) = irf(1,3,1:HOR); iri1acc(tel,1:HOR) = ir(1,3,1:HOR);
                    iri2(tel,1:HOR) = irf(2,3,1:HOR); iri2acc(tel,1:HOR) = ir(2,3,1:HOR);
                    iri3(tel,1:HOR) = irf(3,3,1:HOR); iri3acc(tel,1:HOR) = ir(3,3,1:HOR);
                    iri4(tel,1:HOR) = irf(4,3,1:HOR); iri4acc(tel,1:HOR) = ir(4,3,1:HOR);
                    dei1(tel,1:HOR) = decomp(1,3,1:HOR);
                    dei2(tel,1:HOR) = decomp(2,3,1:HOR);
                    dei3(tel,1:HOR) = decomp(3,3,1:HOR);
                    dei4(tel,1:HOR) = decomp(4,3,1:HOR);
                    sho_mp(tel,1) = strucshock(1,3);
                    A0_mop(tel,:) = a0(:,3);
                end
                
                if s4ident ==1 % stock market illiquidity shock
                    irm1(tel,1:HOR) = irf(1,4,1:HOR); irm1acc(tel,1:HOR) = ir(1,4,1:HOR);
                    irm2(tel,1:HOR) = irf(2,4,1:HOR); irm2acc(tel,1:HOR) = ir(2,4,1:HOR);
                    irm3(tel,1:HOR) = irf(3,4,1:HOR); irm3acc(tel,1:HOR) = ir(3,4,1:HOR);
                    irm4(tel,1:HOR) = irf(4,4,1:HOR); irm4acc(tel,1:HOR) = ir(4,4,1:HOR);
                    dem1(tel,1:HOR) = decomp(1,4,1:HOR);
                    dem2(tel,1:HOR) = decomp(2,4,1:HOR);
                    dem3(tel,1:HOR) = decomp(3,4,1:HOR);
                    dem4(tel,1:HOR) = decomp(4,4,1:HOR);
                    sho_liq(tel,1) = strucshock(1,4);
                    A0_mod(tel,:) = a0(:,4);
                end
                
                if s1ident ==10 && s4ident ==1 % 2,3
                   A0_sup(tel,:)= a0(:,2);
                   A0_dem(tel,:)= a0(:,3);
                   sho_s(tel,1)= strucshock(1,2);
                   sho_d(tel,1)=strucshock(1,3);
                elseif s2ident==10 && s4ident==1 %1,3
                    A0_sup(tel,:)= a0(:,1);
                    A0_dem(tel,:)= a0(:,3);
                    sho_s(tel,1)= strucshock(1,1);
                    sho_d(tel,1)=strucshock(1,3);
                elseif s3ident==10 && s4ident==1 %1,2
                    A0_sup(tel,:)= a0(:,1);
                    A0_dem(tel,:)= a0(:,2);
                    sho_s(tel,1)= strucshock(1,1);
                    sho_d(tel,1)=strucshock(1,2);
                end
                
            end
end
% [count tel]
% line 391 finishes computing the GIRFs satisying the 100 sign restrictions
% imposed on the model.
if count == maxcount
    % empty variables and start again
    sirf=[]; dirf=[]; iirf=[]; mirf=[]; sirfacc=[]; dirfacc=[]; iirfacc=[]; mirfacc=[];......
        fevd1=[]; fevd2=[]; fevd3=[]; fevd4=[];...
        sho_liq=[]; sho_d=[]; sho_s=[]; sho_mp=[];...
        A0_impact=[]; B=[]; ind=0; d=0;
    return
else ind = 1; d = 1;
end


% normalisation of signs to compute distributions 
t1 = irs1(:,1); % supply shock to GDP growth at time 1
t2 = ird1(:,1); % demand shock to GDP growth at time 1
t3 = iri3(:,1); % Monetary Policy Shock to interest rate at time 1
t4 = irm4(:,1); % Liquidity Shock to illiquidity at time 1
vs1 = zeros(tel,1); vs2 = vs1; vs3 = vs1; vs4 = vs1;
for jj = 1:tel
    if t1(jj,1) > 0
        vs1(jj,1) = 1;
    else
        vs1(jj,1) = 1;
    end
    if t2(jj,1) > 0
        vs2(jj,1) = 1;
    else
        vs2(jj,1) = 1;
    end
    if t3(jj,1) >0
        vs3(jj,1) = 1;
    else
        vs3(jj,1) = -1;
    end
    if t4(jj,1) > 0
        vs4(jj,1) = 1;
    else
        vs4(jj,1) = -1;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GIRFs                                                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NON-ACCUMULATED
for jj = 1:HOR
    irs1(:,jj) = vs1.*irs1(:,jj);
    irs2(:,jj) = vs1.*irs2(:,jj);
    irs3(:,jj) = vs1.*irs3(:,jj);
    irs4(:,jj) = vs1.*irs4(:,jj);
end
for jj = 1:HOR
    ird1(:,jj) = vs2.*ird1(:,jj);
    ird2(:,jj) = vs2.*ird2(:,jj);
    ird3(:,jj) = vs2.*ird3(:,jj);
    ird4(:,jj) = vs2.*ird4(:,jj);
end
for jj = 1:HOR
    iri1(:,jj) = vs3.*iri1(:,jj);
    iri2(:,jj) = vs3.*iri2(:,jj);
    iri3(:,jj) = vs3.*iri3(:,jj);
    iri4(:,jj) = vs3.*iri4(:,jj);
end
for jj = 1:HOR
    irm1(:,jj) = vs4.*irm1(:,jj);
    irm2(:,jj) = vs4.*irm2(:,jj);
    irm3(:,jj) = vs4.*irm3(:,jj);
    irm4(:,jj) = vs4.*irm4(:,jj);
end

% Take mean or median of the distribution
% GIRFs to supply shock
girfs1 = mean(irs1); % response of y to supply
girfs2 = mean(irs2); % response of \pi to supply
girfs3 = mean(irs3); % response of i to supply
girfs4 = mean(irs4); % repsonse of m to supply

% GIRFs to demand shock
girfd1 = mean(ird1); % response of y to demand
girfd2 = mean(ird2); % response of \pi to demand
girfd3 = mean(ird3); % response of i to demand
girfd4 = mean(ird4); % repsonse of m to demand

% GIRFs to monetary policy shock
girfi1 = mean(iri1); % response of y to monetary policy 
girfi2 = mean(iri2); % response of \pi to monetary policy 
girfi3 = mean(iri3); % response of i to monetary policy 
girfi4 = mean(iri4); % repsonse of m to monetary policy 

% GIRFs to money demand shock
girfm1 = mean(irm1); % response of y to money demand
girfm2 = mean(irm2); % response of \pi to money demand
girfm3 = mean(irm3); % response of i to money demand
girfm4 = mean(irm4); % repsonse of m to money demand

% Put into matrices for the output
sirf = [girfs1' girfs2' girfs3' girfs4']; 
dirf = [girfd1' girfd2' girfd3' girfd4']; 
iirf = [girfi1' girfi2' girfi3' girfi4']; 
mirf = [girfm1' girfm2' girfm3' girfm4'];

% GIRFs ACCUMULATED
for jj = 1:HOR
    irs1acc(:,jj) = vs1.*irs1acc(:,jj);
    irs2acc(:,jj) = vs1.*irs2acc(:,jj);
    irs3acc(:,jj) = vs1.*irs3acc(:,jj);
    irs4acc(:,jj) = vs1.*irs4acc(:,jj);
end
for jj = 1:HOR
    ird1acc(:,jj) = vs2.*ird1acc(:,jj);
    ird2acc(:,jj) = vs2.*ird2acc(:,jj);
    ird3acc(:,jj) = vs2.*ird3acc(:,jj);
    ird4acc(:,jj) = vs2.*ird4acc(:,jj);
end
for jj = 1:HOR
    iri1acc(:,jj) = vs3.*iri1acc(:,jj);
    iri2acc(:,jj) = vs3.*iri2acc(:,jj);
    iri3acc(:,jj) = vs3.*iri3acc(:,jj);
    iri4acc(:,jj) = vs3.*iri4acc(:,jj);
end
for jj = 1:HOR
    irm1acc(:,jj) = vs4.*irm1acc(:,jj);
    irm2acc(:,jj) = vs4.*irm2acc(:,jj);
    irm3acc(:,jj) = vs4.*irm3acc(:,jj);
    irm4acc(:,jj) = vs4.*irm4acc(:,jj);
end
% Take mean or median of the distribution
% GIRFs to supply shock
girfs1acc = mean(irs1acc); % response of y to supply
girfs2acc = mean(irs2acc); % response of \pi to supply
girfs3acc = mean(irs3acc); % response of i to supply
girfs4acc = mean(irs4acc); % repsonse of m to supply

% GIRFs to demand shock
girfd1acc = mean(ird1acc); % response of y to demand
girfd2acc = mean(ird2acc); % response of \pi to demand
girfd3acc = mean(ird3acc); % response of i to demand
girfd4acc = mean(ird4acc); % repsonse of m to demand

% GIRFs to monetary policy shock
girfi1acc = mean(iri1acc); % response of y to monetary policy 
girfi2acc = mean(iri2acc); % response of \pi to monetary policy 
girfi3acc = mean(iri3acc); % response of i to monetary policy 
girfi4acc = mean(iri4acc); % repsonse of m to monetary policy 

% GIRFs to money demand shock
girfm1acc = mean(irm1acc); % response of y to money demand
girfm2acc = mean(irm2acc); % response of \pi to money demand
girfm3acc = mean(irm3acc); % response of i to money demand
girfm4acc = mean(irm4acc); % repsonse of m to money demand

% Put into matrices for the output
sirfacc = [girfs1acc' girfs2acc' girfs3acc' girfs4acc']; 
dirfacc = [girfd1acc' girfd2acc' girfd3acc' girfd4acc']; 
iirfacc = [girfi1acc' girfi2acc' girfi3acc' girfi4acc']; 
mirfacc = [girfm1acc' girfm2acc' girfm3acc' girfm4acc'];


% Put mean of decompositions into matrix for output
fevd1 = [mean(des1)' mean(des2)' mean(des3)' mean(des4)'];
fevd2 = [mean(ded1)' mean(ded2)' mean(ded3)' mean(ded4)'];
fevd3 = [mean(dei1)' mean(dei2)' mean(dei3)' mean(dei4)'];
fevd4 = [mean(dem1)' mean(dem2)' mean(dem3)' mean(dem4)'];

% Structural shocks
sho_liq = vs4.*sho_liq;
sho_d = vs2.*sho_d;
sho_s = vs1.*sho_s;
sho_mp = vs3.*sho_mp;

sho_liq = mean(sho_liq);
sho_d = mean(sho_d);
sho_s = mean(sho_s);
sho_mp = mean(sho_mp);

% A0 structural impact matrix
for i = 1:N
A0_sup(:,i) = vs1.*A0_sup(:,i);
A0_dem(:,i) = vs2.*A0_dem(:,i);
A0_mop(:,i) = vs3.*A0_mop(:,i);
A0_mod(:,i) = vs4.*A0_mod(:,i);
end

A0_sup = mean(A0_sup);
A0_dem = mean(A0_dem);
A0_mop = mean(A0_mop);
A0_mod = mean(A0_mod);

A0_impact = [A0_sup', A0_dem',A0_mop', A0_mod'];

    


