function [Py, Pp, Pi, Pm, SVDy, SVDp, SVDi, SVDm, SVDy3, SVDp3, SVDi3, SVDm3]...
    = persist_svd(SI, N, L, A0, tel,Tw)

% This function estimates the contributions of structural shocks to
% persistence in each variable within the TVP VAR model and then computes
% the contributions of structural shocks to the overall variability to the
% variables within the model.

% Frequency
omeg = linspace(0,pi,Tw)';
% "i" in spectral density equation sqrt(-1)
i = sqrt(-1);
% Storage variables: Persistence measures
zpy = zeros(tel,N+1);
zpp = zpy;
zpi = zpy;
zpm = zpy;

% Storage Variables: Variance decompositions 2D
svdy = zeros(tel,N);
svdp = svdy;
svdi = svdy;
svdm = svdy;

% Storage Variables: Variance decompositions 3D
svdy3 = zeros(length(omeg),N,tel); 
svdp3 = svdy3;
svdi3 = svdy3;
svdm3 = svdy3;

bet = reshape(SI',1+N*L,N)';
bet = bet(:,2:end);
for jl = 1:L
    BB(:,:,jl) = bet(:,(jl-1)*N+1:(jl-1)*N+N);
end
% L pages means that lagged values of coefficient matrices appear on each
% page of BB.
for kl = 1:tel
    aa0 = A0(:,:,kl);
    VAR = aa0*aa0';
    % Calculate Spectral density
    for w = 1:length(omeg)
        GI = eye(N);
        for al = 1:L
            GI = GI - BB(:,:,al)*exp(-i*al*omeg(w));
        end
        SSS(:,:,w) = (GI\VAR)*inv(GI');
    end
    % shut off everything except supply shock
    a0 = aa0*diag([1 0 0 0 ]');
    VAR = a0*a0';
    for w = 1:length(omeg)
        GI = eye(N);
        for al = 1:L
            GI = GI - BB(:,:,al)*exp(-i*al*omeg(w));
        end
        SS_sup(:,:,w) = (GI\VAR)*inv(GI');
    end
    % shut off everything except demand shock
    a0 = aa0*diag([0 1 0 0]');
    VAR = a0*a0';
    for w = 1:length(omeg)
        GI = eye(N);
        for al = 1:L
            GI = GI - BB(:,:,al)*exp(-i*al*omeg(w));
        end
        SS_dem(:,:,w) = (GI\VAR)*inv(GI');
    end
    % shut off everything except monetary policy shock
    a0 = aa0*diag([0 0 1 0]');
    VAR = a0*a0';
    for w = 1:length(omeg)
        GI = eye(N);
        for al = 1:L
            GI = GI - BB(:,:,al)*exp(-i*al*omeg(w));
        end
        SS_mop(:,:,w) = (GI\VAR)*inv(GI');
    end
    % shut off everything except money demand shock
    a0 = aa0*diag([0 0 0 1]');
    VAR = a0*a0';
    for w = 1:length(omeg)
        GI = eye(N);
        for al = 1:L
            GI = GI - BB(:,:,al)*exp(-i*al*omeg(w));
        end
        SS_mod(:,:,w) = (GI\VAR)*inv(GI');
    end
    % PERSISTENCE CALCULATIONS AND CONTRIBUTIONS OF STRUCTURAL SHOCKS TO
    % PERSISTENCE OF VARIABLES
    % 1st variable is y_{t} gdp growth
    specy = real(squeeze(SSS(1,1,:))); % overall spectral density
    nspecy = specy/sum(specy);         %  normalised spec = spec/integral(spec) => integral(spec) is variance
    nspecyS = real(squeeze(SS_sup(1,1,:)))/sum(specy);
    nspecyD = real(squeeze(SS_dem(1,1,:)))/sum(specy);
    nspecyI = real(squeeze(SS_mop(1,1,:)))/sum(specy);
    nspecyM = real(squeeze(SS_mod(1,1,:)))/sum(specy);
    zpy(kl,:) = [nspecy(1) [nspecyS(1) nspecyD(1) nspecyI(1) nspecyM(1)]/nspecy(1)];
    % Py is the persistence of the first variable in the first column and the remaining
    % columns are the contributions of the structural shocks to the
    % persistence of the variable in question. We want the first
    % observation because this represents the estimate at omeg=0 or the
    % infinite horizon variability.
    
    % 2nd variable is \pi_{t} inflation
    specp = real(squeeze(SSS(2,2,:))); % overall spectral density
    nspecp = specp/sum(specp);         %  normalised spec = spec/integral(spec) => integral(spec) is variance
    nspecpS = real(squeeze(SS_sup(2,2,:)))/sum(specp);
    nspecpD = real(squeeze(SS_dem(2,2,:)))/sum(specp);
    nspecpI = real(squeeze(SS_mop(2,2,:)))/sum(specp);
    nspecpM = real(squeeze(SS_mod(2,2,:)))/sum(specp);
    zpp(kl,:) = [nspecp(1) [nspecpS(1) nspecpD(1) nspecpI(1) nspecpM(1)]/nspecp(1)];
    
    % 3rd variable is i_{t} interest rate
    speci = real(squeeze(SSS(3,3,:))); % overall spectral density
    nspeci = speci/sum(speci);         %  normalised spec = spec/integral(spec) => integral(spec) is variance
    nspeciS = real(squeeze(SS_sup(3,3,:)))/sum(speci);
    nspeciD = real(squeeze(SS_dem(3,3,:)))/sum(speci);
    nspeciI = real(squeeze(SS_mop(3,3,:)))/sum(speci);
    nspeciM = real(squeeze(SS_mod(3,3,:)))/sum(speci);
    zpi(kl,:) = [nspeci(1) [nspeciS(1) nspeciD(1) nspeciI(1) nspeciM(1)]/nspeci(1)];
    
    % 4th variable is m_{t} money growth
    specm = real(squeeze(SSS(4,4,:))); % overall spectral density
    nspecm = specm/sum(specm);         %  normalised spec = spec/integral(spec) => integral(spec) is variance
    nspecmS = real(squeeze(SS_sup(4,4,:)))/sum(specm);
    nspecmD = real(squeeze(SS_dem(4,4,:)))/sum(specm);
    nspecmI = real(squeeze(SS_mop(4,4,:)))/sum(specm);
    nspecmM = real(squeeze(SS_mod(4,4,:)))/sum(specm);
    zpm(kl,:) = [nspecm(1) [nspecmS(1) nspecmD(1) nspecmI(1) nspecmM(1)]/nspecm(1)];
    
    % VARIANCE DECOMPOSITIONS FOR VARIABLES 2D so we sum over the
    % frequencies
    
    % 1st variable y_{t}
    vdyS = sum(real(squeeze(SS_sup(1,1,:))))/sum(specy);
    vdyD = sum(real(squeeze(SS_dem(1,1,:))))/sum(specy);
    vdyI = sum(real(squeeze(SS_mop(1,1,:))))/sum(specy);
    vdyM = sum(real(squeeze(SS_mod(1,1,:))))/sum(specy);
    svdy(kl,:) = [vdyS vdyD vdyI vdyM];
    
    % 2nd variable \pi_{t}
    vdpS = sum(real(squeeze(SS_sup(2,2,:))))/sum(specp);
    vdpD = sum(real(squeeze(SS_dem(2,2,:))))/sum(specp);
    vdpI = sum(real(squeeze(SS_mop(2,2,:))))/sum(specp);
    vdpM = sum(real(squeeze(SS_mod(2,2,:))))/sum(specp);
    svdp(kl,:) = [vdpS vdpD vdpI vdpM];
    
    % 3rd variable i_{t}
    vdiS = sum(real(squeeze(SS_sup(3,3,:))))/sum(speci);
    vdiD = sum(real(squeeze(SS_dem(3,3,:))))/sum(speci);
    vdiI = sum(real(squeeze(SS_mop(3,3,:))))/sum(speci);
    vdiM = sum(real(squeeze(SS_mod(3,3,:))))/sum(speci);
    svdi(kl,:) = [vdiS vdiD vdiI vdiM];
    
    % 4th variable m_{t}
    vdmS = sum(real(squeeze(SS_sup(4,4,:))))/sum(specm);
    vdmD = sum(real(squeeze(SS_dem(4,4,:))))/sum(specm);
    vdmI = sum(real(squeeze(SS_mop(4,4,:))))/sum(specm);
    vdmM = sum(real(squeeze(SS_mod(4,4,:))))/sum(specm);
    svdm(kl,:) = [vdmS vdmD vdmI vdmM]; % svdm is telxN
    
    % VARIANCE DECOMPOSITIONS FOR VARIABLES 3D
    % 1st variable
    vdyS3 = real(squeeze(SS_sup(1,1,:)))./specy; % omegx1
    vdyD3 = real(squeeze(SS_dem(1,1,:)))./specy;
    vdyI3 = real(squeeze(SS_mop(1,1,:)))./specy;
    vdyM3 = real(squeeze(SS_mod(1,1,:)))./specy;
    svdy3(:,:,kl) = [vdyS3 vdyD3 vdyI3 vdyM3];
    
    % 2nd variable
    vdpS3 = real(squeeze(SS_sup(2,2,:)))./specp; % omegx1
    vdpD3 = real(squeeze(SS_dem(2,2,:)))./specp;
    vdpI3 = real(squeeze(SS_mop(2,2,:)))./specp;
    vdpM3 = real(squeeze(SS_mod(2,2,:)))./specp;
    svdp3(:,:,kl) = [vdpS3 vdpD3 vdpI3 vdpM3];
    
    % 3rd variable
    vdiS3 = real(squeeze(SS_sup(3,3,:)))./speci; % omegx1
    vdiD3 = real(squeeze(SS_dem(3,3,:)))./speci;
    vdiI3 = real(squeeze(SS_mop(3,3,:)))./speci;
    vdiM3 = real(squeeze(SS_mod(3,3,:)))./speci;
    svdi3(:,:,kl) = [vdiS3 vdiD3 vdiI3 vdiM3];
    
    % 4th variable
    vdmS3 = real(squeeze(SS_sup(4,4,:)))./specm; % omegx1
    vdmD3 = real(squeeze(SS_dem(4,4,:)))./specm;
    vdmI3 = real(squeeze(SS_mop(4,4,:)))./specm;
    vdmM3 = real(squeeze(SS_mod(4,4,:)))./specm;
    svdm3(:,:,kl) = [vdmS3 vdmD3 vdmI3 vdmM3];
end   
% Take mean of variables along tel
% PERSISTENCE
Py = mean(zpy,1); Pp = mean(zpp,1); Pi = mean(zpi,1); Pm = mean(zpm,1);
% SVD 2D
SVDy = mean(svdy,1); SVDp = mean(svdp,1); SVDi = mean(svdi,1); SVDm = mean(svdm,1);
% SVD 3D
SVDy3 = mean(svdy3,3); SVDp3 = mean(svdp3,3); SVDi3 = mean(svdi3,3); SVDm3 = mean(svdm3,3);
% the 3D variance decompositions should be omega by N, columns are
% contribution of structural shock to spectral density.




