function [TIME, NN, TT] = TVP_IRF(DFILE2, L, N, TP, time, NG, data, DFILE3, h, SC, R, G1,ident)

load(DFILE2)

NN = NG-1; % Gibbs draws
T = size(SD,2); % Second dimension of SD (i.e. theta matrices) is the number of time periods
[YS, XS1, time] = databasics(time, L, TP, data); % load data and time vector
clear DD II UU
% Variables I want to save:
% Must be the same length in order to store strings in matrix
% FULL DISTRIBUTIONS
varname(1,:) = 'IRFP'; % productivity
varname(2,:) = 'IRFJ'; % job sep
varname(3,:) = 'IRFW'; % wage 
varname(4,:) = 'IRFD'; % demand

% FORECAST ERROR VARIANCE DECOMPOSITIONS FULL DISTRIBUTIONS
varname(5,:) = 'FVDP';
varname(6,:) = 'FVDJ';
varname(7,:) = 'FVDW';
varname(8,:) = 'FVDD';

% Structural Shocks
varname(9,:) = 'SSPR';
varname(10,:) = 'SSJS';
varname(11,:) = 'SSWA';
varname(12,:) = 'SSWA';

varname(13,:) = 'AA00';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Impulse Response functions based on sign restrictions
HOR = 41; % Impulse horizon number of steps = 20
cors = 1; % Identification Parameter, 1 = contemporaneous
NMCInteg = 100; % Number of Monte Carlo Integrations to get GIRFs SET TO 100
TT = (1:1:T)';
% If wish to sample only every fourth quarter (speed up process) comment
% above and uncomment below:
% TT = (1:4:T)';
TT = TT(1+L:length(TT));
epsilon = shockhist(T, N, L, YS, XS1, SD); 
% only need shockhist for historical decomposition change line 8 to include
% XS1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Define storage matrices
IRFP = zeros(HOR,N,R,length(TT));
IRFJ = zeros(HOR,N,R,length(TT));
IRFW = zeros(HOR,N,R,length(TT));
IRFD = IRFP;

 % ACCUMULATED DISTRIBUTIONS

FVDP = zeros(HOR,N,R,length(TT)); % FEVD over HOR for each variable to SUPPLY SHOCK
FVDJ = FVDP;
FVDW = FVDP;
FVDD = FVDP;

SSPR = zeros(R,length(TT));
SSJS = zeros(R,length(TT));
SSWA = SSPR;
SSDE = SSPR;
AA00=zeros(N,N,R,length(TT));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% GIRFs following Koop et al (1996) and Rubio Ramirez et al (2010)
% Draw 500 states of the economy at each point in time randomly and search
% for 100 impulse responses that satisfy the sign restrictions for each
% state

kk = 1
tic;
while kk <= length(TT) % kk indexes time here
    toc
    tt = TT(kk); 
    IRFPP = zeros(HOR, N, R); % Impulses in each period
    IRFJJ = zeros(HOR, N, R);
    IRFWW = zeros(HOR, N, R);
    IRFDD = zeros(HOR, N, R);
    
    
    FEV1 = zeros(HOR, N, R); % FEVD
    FEV2 = zeros(HOR, N, R);
    FEV3 = zeros(HOR, N, R);
    FEV4 = zeros(HOR, N, R);
    sspr = zeros(1,R);
    ssjs = sspr;
    sswa = sspr;
    ssde = sspr;
    AA0=zeros(N,N,R);
    
    hh = 1;
    while hh <= R; % Replications of the economy drawn randomly
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Sample state of the economy from the Gibbs output
        K = fix(rand*(NN-0.01))+1; % Random draw from the maximum range of Gibbs draws
        sd = SD(:,tt,K); % Theta_{t} time-varying coefficients. SD is N*(1+N*L)xTxNG
        aa = AA(:,tt,K); % A_{t} elements \alpha_{i,t}. AA is (N*(N-1))/2xTxNG
        if G1==1
        qd=QD(:,:,K);
        qv=0;
        elseif G1==0
        qd = QD(tt,:,K)';  % varies over time. matrix Q. QD is TxN*(1+N*L)xNG
        qv = QV(:,K);
        end
        s1 = S1(1,K);    % S1 1xNG
        s2 = S2(:,:,K);  % S2 2x2xNG
        s3 = S3(:,:,K);
        s4 = S4(:,:,K);
        vd = VD(:,K);    % W matrix or standard deviation of volatility innovation NxNG
        om = OM(tt,:,K)'; % H_{t} diagonal elements stochastic volatilities TxNxNG
        
        if ident==0
            [pirf, jirf, wirf, dirf, fevd1, fevd2, fevd3, fevd4, shp, shj, shw, shd, A0, ind, d]...
                = GIRF_FEVD_2019(sd, aa, qd, qv, s1, s2, s3, s4, vd, om,...
    YS(:,tt-L:tt-1), HOR, N, L, cors, NMCInteg, h, epsilon(tt,:,K), SC,G1);
        elseif ident==1
            [pirf, jirf, wirf, dirf, fevd1, fevd2, fevd3, fevd4, shp, shj, shw, shd, A0, ind, d]...
                = GIRF_FEVD_ALT(sd, aa, qd, qv, s1, s2, s3, s4, vd, om,...
    YS(:,tt-L:tt-1), HOR, N, L, cors, NMCInteg, h, epsilon(tt,:,K), SC,G1);
        end

        if ind == 1 && d == 1
            % GIRFs
            IRFPP(:,:,hh) = pirf;
            IRFJJ(:,:,hh) = jirf;
            IRFWW(:,:,hh) = wirf;
            IRFDD(:,:,hh) = dirf;
            
            AA0(:,:,hh)=A0;

            % FEVDs
            FEV1(:,:,hh) = fevd1; FEV2(:,:,hh) = fevd2; FEV3(:,:,hh) = fevd3; FEV4(:,:,hh) = fevd4;
            sspr(:,hh) = shp; ssjs(:,hh) = shj; sswa(:,hh) = shw; ssde(:,hh)=shd;
            hh = hh + 1;
        elseif ind == 0 && d == 0
            hh;
        end
        clear pirf jirf wirf fevd1 fevd2 fevd3 shp shj shw ind d
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % FULL DISTRIBUTIONS                                                  %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Impulse Responses over time: NON ACCUMULATED
    IRFP(:,:,:,kk) = IRFPP; % Response of variables to supply shock (HORxNxRxT);
    IRFJ(:,:,:,kk) = IRFJJ; % Response of variables to monetary policy shock (HORxNxRxT);
    IRFW(:,:,:,kk) = IRFWW;
    IRFD(:,:,:,kk) = IRFDD;

    % FORECAST ERROR VARIANCE DECOMPOSITIONS
    FVDP(:,:,:,kk) = FEV1; FVDJ(:,:,:,kk) = FEV2; FVDW(:,:,:,kk) = FEV3; FVDD(:,:,:,kk)=FEV4;
    
    
    SSPR(:,kk) = sspr';
    SSJS(:,kk) = ssjs';
    SSWA(:,kk) = sswa';
    SSDE(:,kk) = ssde';
    AA00(:,:,:,kk) = AA0;
    kk = kk + 1
    save(DFILE3, varname(1,:), varname(2,:), varname(3,:), varname(4,:), varname(5,:),...
    varname(6,:), varname(7,:), varname(8,:), varname(9,:), varname(10,:), varname(11,:), varname(12,:),...
    varname(13,:));
end
TIME = time(1+L:length(TT));
